Inspector = {
	VERSION = 190110,
	configFile = nil,
	color = {
		owned = {
			normal = {1.0, 1.0, 1.0, 1.0},
			warning = {1.0, 1.0, 0.0, 1.0},
			urgent = {1.0, 0.0, 0.0, 1.0}
		},
		other = {
			normal = {0.749, 0.0, 0.749, 1.0},
			warning = {1.0, 1.0, 0.0, 1.0},
			urgent = {1.0, 0.0, 0.0, 1.0}
		}
	},
	pos = {0.785, 0.87},
	stringLength = 40,
	font = {
		size = 0.011,
		line = 1.2
	},
	limits = {
		normal = { low = 30, high = 70 },
		warning = { low = 20, high = 80 },
		urgent = { low = 10, high = 90 }
	},
	display = {
		variant = 0,
		MAX_variant = 3,
		VARIANT_NORMAL = 0,
		VARIANT_FULL = 1,
		VARIANT_SHORT = 2
	},
	AI = {
		blocked = {
			urgent = true
		}
	}
};

function Inspector:loadMap(name)
	print("--- loading Fruchtzwerg's Inspector ---")
	local modSettingsDir = getUserProfileAppPath() .. "modsSettings/";
	createFolder(modSettingsDir);
	local mySettingsDir = modSettingsDir .. "Inspector/";
	createFolder(mySettingsDir);
	Inspector.configFile = mySettingsDir .. "Inspector.xml";
	print("Using config file " .. Inspector.configFile);
	if fileExists(Inspector.configFile) then
		Inspector:loadConfig();
	else
		Inspector:saveConfig();
	end
end

function Inspector:deleteMap()
	Inspector:saveConfig();
end

function Inspector:loadConfig()
	if Inspector.configFile == nil then
		return;
	end
	local xml = loadXMLFile("Inspector_XML", Inspector.configFile, "Inspector");
	local xmlVersion = Utils.getNoNil(getXMLInt(xml, "Inspector#VERSION"), 0);
	if xmlVersion > 0 then
		if xmlVersion >= 190110 and xmlVersion <= Inspector.VERSION then
			Inspector.color.owned.normal[1] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.owned.normal#R"), Inspector.color.owned.normal[1]);
			Inspector.color.owned.normal[2] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.owned.normal#G"), Inspector.color.owned.normal[2]);
			Inspector.color.owned.normal[3] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.owned.normal#B"), Inspector.color.owned.normal[3]);
			Inspector.color.owned.normal[4] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.owned.normal#A"), Inspector.color.owned.normal[4]);
			Inspector.color.owned.warning[1] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.owned.warning#R"), Inspector.color.owned.warning[1]);
			Inspector.color.owned.warning[2] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.owned.warning#G"), Inspector.color.owned.warning[2]);
			Inspector.color.owned.warning[3] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.owned.warning#B"), Inspector.color.owned.warning[3]);
			Inspector.color.owned.warning[4] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.owned.warning#A"), Inspector.color.owned.warning[4]);
			Inspector.color.owned.urgent[1] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.owned.urgent#R"), Inspector.color.owned.urgent[1]);
			Inspector.color.owned.urgent[2] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.owned.urgent#G"), Inspector.color.owned.urgent[2]);
			Inspector.color.owned.urgent[3] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.owned.urgent#B"), Inspector.color.owned.urgent[3]);
			Inspector.color.owned.urgent[4] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.owned.urgent#A"), Inspector.color.owned.urgent[4]);
			Inspector.color.other.normal[1] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.other.normal#R"), Inspector.color.other.normal[1]);
			Inspector.color.other.normal[2] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.other.normal#G"), Inspector.color.other.normal[2]);
			Inspector.color.other.normal[3] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.other.normal#B"), Inspector.color.other.normal[3]);
			Inspector.color.other.normal[4] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.other.normal#A"), Inspector.color.other.normal[4]);
			Inspector.color.other.warning[1] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.other.warning#R"), Inspector.color.other.warning[1]);
			Inspector.color.other.warning[2] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.other.warning#G"), Inspector.color.other.warning[2]);
			Inspector.color.other.warning[3] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.other.warning#B"), Inspector.color.other.warning[3]);
			Inspector.color.other.warning[4] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.other.warning#A"), Inspector.color.other.warning[4]);
			Inspector.color.other.urgent[1] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.other.urgent#R"), Inspector.color.other.urgent[1]);
			Inspector.color.other.urgent[2] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.other.urgent#G"), Inspector.color.other.urgent[2]);
			Inspector.color.other.urgent[3] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.other.urgent#B"), Inspector.color.other.urgent[3]);
			Inspector.color.other.urgent[4] = Utils.getNoNil(getXMLFloat(xml, "Inspector.color.other.urgent#A"), Inspector.color.other.urgent[4]);
			Inspector.limits.normal.low = Utils.getNoNil(getXMLFloat(xml, "Inspector.limits#normal"), Inspector.limits.normal.low);
			Inspector.limits.warning.low = Utils.getNoNil(getXMLFloat(xml, "Inspector.limits#warning"), Inspector.limits.warning.low);
			Inspector.limits.urgent.low = Utils.getNoNil(getXMLFloat(xml, "Inspector.limits#urgent"), Inspector.limits.urgent.low);
			Inspector.pos[1] = Utils.getNoNil(getXMLFloat(xml, "Inspector.position#x"), Inspector.pos[1]);
			Inspector.pos[2] = Utils.getNoNil(getXMLFloat(xml, "Inspector.position#y"), Inspector.pos[2]);
			Inspector.stringLength = Utils.getNoNil(getXMLInt(xml, "Inspector.string#length"), Inspector.stringLength);
			Inspector.font.size = Utils.getNoNil(getXMLFloat(xml, "Inspector.font#size"), Inspector.font.size);
			Inspector.font.line = Utils.getNoNil(getXMLFloat(xml, "Inspector.font#line"), Inspector.font.line);
			Inspector.display.variant = Utils.getNoNil(getXMLInt(xml, "Inspector.display#variant"), Inspector.display.variant);
			Inspector.AI.blocked.urgent = Utils.getNoNil(getXMLBool(xml, "Inspector.AI.blocked#urgent"), Inspector.AI.blocked.urgent);
		end
	end
	if delete ~= nil then
		delete(xml);
	end
	if xmlVersion ~= Inspector.VERSION then
		Inspector:saveConfig();
	end

	Inspector.limits.normal.high = 100 - Inspector.limits.normal.low;
	Inspector.limits.warning.high = 100 - Inspector.limits.warning.low;
	Inspector.limits.urgent.high = 100 - Inspector.limits.urgent.low;
	
end

function Inspector:saveConfig()
	if Inspector.configFile == nil then
		return;
	end
	local xml = createXMLFile("Inspector_XML", Inspector.configFile, "Inspector");
	setXMLInt(xml, "Inspector#VERSION", Inspector.VERSION);
	setXMLFloat(xml, "Inspector.color.owned.normal#R", Inspector.color.owned.normal[1]);
	setXMLFloat(xml, "Inspector.color.owned.normal#G", Inspector.color.owned.normal[2]);
	setXMLFloat(xml, "Inspector.color.owned.normal#B", Inspector.color.owned.normal[3]);
	setXMLFloat(xml, "Inspector.color.owned.normal#A", Inspector.color.owned.normal[4]);
	setXMLFloat(xml, "Inspector.color.owned.warning#R", Inspector.color.owned.warning[1]);
	setXMLFloat(xml, "Inspector.color.owned.warning#G", Inspector.color.owned.warning[2]);
	setXMLFloat(xml, "Inspector.color.owned.warning#B", Inspector.color.owned.warning[3]);
	setXMLFloat(xml, "Inspector.color.owned.warning#A", Inspector.color.owned.warning[4]);
	setXMLFloat(xml, "Inspector.color.owned.urgent#R", Inspector.color.owned.urgent[1]);
	setXMLFloat(xml, "Inspector.color.owned.urgent#G", Inspector.color.owned.urgent[2]);
	setXMLFloat(xml, "Inspector.color.owned.urgent#B", Inspector.color.owned.urgent[3]);
	setXMLFloat(xml, "Inspector.color.owned.urgent#A", Inspector.color.owned.urgent[4]);
	setXMLFloat(xml, "Inspector.color.other.normal#R", Inspector.color.other.normal[1]);
	setXMLFloat(xml, "Inspector.color.other.normal#G", Inspector.color.other.normal[2]);
	setXMLFloat(xml, "Inspector.color.other.normal#B", Inspector.color.other.normal[3]);
	setXMLFloat(xml, "Inspector.color.other.normal#A", Inspector.color.other.normal[4]);
	setXMLFloat(xml, "Inspector.color.other.warning#R", Inspector.color.other.warning[1]);
	setXMLFloat(xml, "Inspector.color.other.warning#G", Inspector.color.other.warning[2]);
	setXMLFloat(xml, "Inspector.color.other.warning#B", Inspector.color.other.warning[3]);
	setXMLFloat(xml, "Inspector.color.other.warning#A", Inspector.color.other.warning[4]);
	setXMLFloat(xml, "Inspector.color.other.urgent#R", Inspector.color.other.urgent[1]);
	setXMLFloat(xml, "Inspector.color.other.urgent#G", Inspector.color.other.urgent[2]);
	setXMLFloat(xml, "Inspector.color.other.urgent#B", Inspector.color.other.urgent[3]);
	setXMLFloat(xml, "Inspector.color.other.urgent#A", Inspector.color.other.urgent[4]);
	setXMLFloat(xml, "Inspector.limits#normal", Inspector.limits.normal.low);
	setXMLFloat(xml, "Inspector.limits#warning", Inspector.limits.warning.low);
	setXMLFloat(xml, "Inspector.limits#urgent", Inspector.limits.urgent.low);
	setXMLFloat(xml, "Inspector.position#x", Inspector.pos[1]);
	setXMLFloat(xml, "Inspector.position#y", Inspector.pos[2]);
	setXMLInt(xml, "Inspector.string#length", Inspector.stringLength);
	setXMLFloat(xml, "Inspector.font#size", Inspector.font.size);
	setXMLFloat(xml, "Inspector.font#line", Inspector.font.line);
	setXMLInt(xml, "Inspector.display#variant", Inspector.display.variant);
	setXMLBool(xml, "Inspector.AI.blocked#urgent", Inspector.AI.blocked.urgent);
	saveXMLFile(xml);
	if delete ~= nil then
		delete(xml);
	end
end

function Inspector:mouseEvent(posX, posY, isDown, isUp, button)
end

function Inspector:keyEvent(unicode, sym, modifier, isDown)	
end

function Inspector:update(dt)
end

function Inspector:registerActionEvents()
	local r2, eventName1 = g_inputBinding:registerActionEvent('Inspector_reload_xml', self, Inspector.actionReloadXML, false, true, false, true)
	g_inputBinding.events[eventName1].displayIsVisible = false;
	local r2, eventName2 = g_inputBinding:registerActionEvent('Inspector_mode_switch', self, Inspector.actionModeSwitch, false, true, false, true)
	g_inputBinding.events[eventName2].displayIsVisible = false;
end

function Inspector.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onAIStart", Inspector);
	SpecializationUtil.registerEventListener(vehicleType, "onAIBlock", Inspector);
	SpecializationUtil.registerEventListener(vehicleType, "onAIContinue", Inspector);
end

function Inspector:actionReloadXML()
	Inspector:loadConfig();
end

function Inspector:actionModeSwitch()
	Inspector.display.variant = (Inspector.display.variant + 1) % Inspector.display.MAX_variant;
end

function Inspector:draw(dt)
	if not g_currentMission.isLoaded or g_currentMission.player == nil then
		return;
	end
	setTextAlignment(RenderText.ALIGN_LEFT)
	--setTextColor(unpack(Inspector.color));
	self.nLine = 0;
	self.visited = {};
	local buffer = { {}, {}, {}, {}, {} };
	for i,vehicle in pairs(g_currentMission.vehicles) do
		self.visiting = {};
		self:drawVehicle(vehicle, buffer);
	end
	for bufferIndex,bufferPart in ipairs(buffer) do
		local keys = {};
		for k,v in pairs(bufferPart) do
			table.insert(keys, k);
		end
		table.sort(keys);
		for _,key in ipairs(keys) do
			for i,info in ipairs(bufferPart[key]) do
				self:renderVehicleInfo(info);
			end
		end
	end
	self.visiting = nil;
	self.visited = nil
	--respect settings for other mods
	setTextAlignment(0);
	setTextColor(1, 1, 1, 1);
	setTextBold(false);
	--respect settings for other mods
end

function Inspector:drawVehicle(vehicle, buffer)
	if self.visited[vehicle] ~= nil then
		return true;
	end
	if self.visiting[vehicle] ~= nil then
		return false;
	end
	self.visiting[vehicle] = true
	
	if vehicle.getAttacherVehicle ~= nil then
		local attacherVehicle = vehicle:getAttacherVehicle();
		if attacherVehicle ~= nil and attacherVehicle ~= vehicle then
			if self:drawVehicle(attacherVehicle, buffer) then
				return true;
			end
		end
	end
	
	local info = self:getVehicleInfo(vehicle, false);
	if info.draw then
		local bufferIndex = 1;
		if vehicle.typeName == "pallet" then
			bufferIndex = 3;
		elseif info.owned then
			if vehicle.spec_enterable ~= nil then
				bufferIndex = 1;
			else
				bufferIndex = 2;
			end
		else
			if vehicle.spec_enterable ~= nil then
				bufferIndex = 4;
			else
				bufferIndex = 5;
			end
		end
		if buffer[bufferIndex][info.name] == nil then
			buffer[bufferIndex][info.name] = {};
		end
		table.insert(buffer[bufferIndex][info.name], info);
	end
	return true;
end

function Inspector:getVehicleInfo(vehicle, isPrimaryControlled)
	self.visited[vehicle] = true;
	local primaryLine = self:getVehicleLine(vehicle, isPrimaryControlled);
	local draw, parts, isControlled, isSelfControlled, isFarmControlled, owned = primaryLine.draw, { primaryLine }, primaryLine.isControlled, primaryLine.isSelfControlled, primaryLine.isFarmControlled, primaryLine.owned;
	if vehicle.getAttachedImplements ~= nil then
		for i,implement in pairs(vehicle:getAttachedImplements()) do
			if implement ~= nil and implement.object ~= nil then
				local subInfo = self:getVehicleInfo(implement.object, isPrimaryControlled or primaryLine.isControlled);
				for i,subPart in ipairs(subInfo.parts) do
					table.insert(parts, subPart);
				end
				draw = draw or subInfo.draw;
				isControlled = isControlled or subInfo.isControlled;
				isSelfControlled = isSelfControlled or subInfo.isSelfControlled;
				isFarmControlled = isFarmControlled or subInfo.isFarmControlled;
				owned = owned or subInfo.owned;
			end
		end
	end
	return {
		draw = draw,
		parts = parts,
		isControlled = isControlled,
		isSelfControlled = isSelfControlled,
		isFarmControlled = isFarmControlled,
		owned = owned,
		name = primaryLine.name,
		isAIBlocked = primaryLine.isAIBlocked
	};
end

function Inspector:getControllerFarmId(controller)
	for k,player in pairs(g_currentMission.players) do
		if player.visualInformation.playerName == controller then
			return player.farmId;
		end
	end
	return nil;
end

function Inspector:getVehicleLine(vehicle, isPrimaryControlled)
	local isHired, isMotorized, isMotorStarted, isAIBlocked = false, false, false, false;
	local isSelfControlled, isFarmControlled, controller, helperName = false, false, nil, nil;
	local owned = (vehicle.ownerFarmId == g_currentMission.player.farmId);
	if vehicle.getIsControlled ~= nil and vehicle:getIsControlled() then
		isSelfControlled = g_currentMission.controlledVehicle == vehicle;
		controller = vehicle:getControllerName();
		if isSelfControlled then
			isFarmControlled = isSelfControlled
		elseif controller ~= nil then
			local controllerFarmId = self:getControllerFarmId(controller);
			isFarmControlled = controllerFarmId ~= nil and controllerFarmId == g_currentMission.player.farmId;
		end
	end
	if vehicle.getIsMotorStarted ~= nil then
		isMotorized = true;
		isMotorStarted = vehicle:getIsMotorStarted();
		if vehicle.getIsAIActive ~= nil and vehicle:getIsAIActive() then
			helperName = "*";
			if vehicle.getCurrentHelper ~= nil then
				local currentHelper = vehicle:getCurrentHelper();
				if currentHelper ~= nil and currentHelper.name ~= nil then
					helperName = string.sub(tostring(currentHelper.name) .. "*", 0, 1);
				end
			end
			isAIBlocked = vehicle.Inspector_AIblocked or false;
		end
	end

	local draw, actor, isControlled = isSelfControlled or isFarmControlled, nil, controller ~= nil;

	if controller ~= nil then
		actor = string.sub(controller, 0, Inspector.stringLength);
		draw = draw or owned;
	elseif helperName ~= nil then
		draw = draw or owned;
		isControlled = isControlled or owned;
		isFarmControlled = isFarmControlled or owned;
	elseif isMotorStarted then
		draw = draw or owned;
		isControlled = isControlled or owned;
		isFarmControlled = isFarmControlled or owned;
		actor = "!";
	end
	
	local drawFill, fill = self:getVehicleFill(vehicle, isControlled or isPrimaryControlled);
	
	if helperName ~= nil then
		if actor ~= nil then
			actor = actor .. " (" .. helperName .. ")";
		else
			actor = "(" .. helperName .. ")";
		end
	end
	
	draw = draw or ((owned or isFarmControlled) and drawFill) or isAIBlocked;
	if Inspector.display.variant == Inspector.display.VARIANT_SHORT then
		draw = draw and (isControlled or isFarmControlled or isPrimaryControlled);
	end
	
	return {
		draw = draw,
		owned = owned,
		actor = actor,
		name = vehicle:getName(),
		fill = fill,
		isControlled = isControlled,
		isSelfControlled = isSelfControlled,
		isFarmControlled = isFarmControlled,
		isAIBlocked = isAIBlocked
	};
end

function Inspector:getFillTypeName(id)
	local fillType = g_currentMission.fillTypeManager.fillTypes[id];
	local name = tostring(fillType.title or fillType.name);
	return name;
end

function Inspector:getFillUnitPercentage(unit)
	if unit == nil then
		return 0;
	end
	if unit.capacity > 0 then
		return 100 * unit.fillLevel / unit.capacity;
	end
	return nil;
end

function Inspector:getFillInfo(name, percentage, drawOnHigh, warnOnHigh, warningLevelFactor, isMachineFill)
	local draw, bold, warning, urgent = false, false, false, false;
	if drawOnHigh ~= nil then
		local percentageH, percentageL = percentage * warningLevelFactor, percentage / warningLevelFactor;
		if drawOnHigh then
			if percentageH >= Inspector.limits.normal.high then
				draw = true;
			end
		else
			if percentageL <= Inspector.limits.normal.low then
				draw = true;
			end
		end
		if warnOnHigh ~= nil then
			if warnOnHigh then
				if percentageH >= Inspector.limits.urgent.high then
					draw = true;
					bold = true;
					urgent = true;
				elseif percentageH >= Inspector.limits.warning.high then
					draw = true;
					bold = true;
					warning = true;
				elseif percentageH >= Inspector.limits.normal.high then
					draw = true;
				end
			else
				if percentageL <= Inspector.limits.urgent.low then
					draw = true;
					bold = true;
					urgent = true;
				elseif percentageL <= Inspector.limits.warning.low then
					draw = true;
					bold = true;
					warning = true;
				elseif percentageL <= Inspector.limits.normal.low then
					draw = true;
				end
			end
		end
	end
	return {
		name = string.sub(name, 0, Inspector.stringLength),
		percentage = percentage,
		draw = draw,
		bold = bold,
		warning = warning,
		urgent = urgent,
		isMachineFill = isMachineFill or false
	};
end

function Inspector:getVehicleFill(vehicle, isPrimaryControlled)
	local draw, fill = false, {};

	if vehicle.getVehicleDamage ~= nil then
		local damage = vehicle:getVehicleDamage();
		if damage ~= nil then
			local condition = 100 - (damage * 100);
			if condition > 100 then
				condition = 100;
			elseif condition < 0 then
				condition = 0;
			end
			local info = self:getFillInfo(g_i18n:getText("text_Inspector_Condition"), condition, false, false, 1.5);
			if Inspector.display.variant == Inspector.display.VARIANT_FULL then
				info.draw = true;
			end
			table.insert(fill, info);
			draw = draw or info.draw;
		end
	end

	local isMotorized, isSprayer, isSowingMachine, isCombine = vehicle.getIsMotorStarted ~= nil, SpecializationUtil.hasSpecialization(Sprayer, vehicle.specializations), SpecializationUtil.hasSpecialization(SowingMachine, vehicle.specializations), SpecializationUtil.hasSpecialization(Combine, vehicle.specializations);
	
	if vehicle.getFillUnits ~= nil then
		local skipFillUnit = {};
		if isMotorized then
			local spec = vehicle.spec_motorized;
			if spec.consumersByFillTypeName.diesel ~= nil then
				local index = spec.consumersByFillTypeName.diesel.fillUnitIndex;
				skipFillUnit[index] = true;
				local unit = vehicle:getFillUnitByIndex(index);
				local percentage = self:getFillUnitPercentage(unit);
				if percentage ~= nil then
					local fillType = FillType.DIESEL;
					if unit ~= nil and unit.fillType > 1 then
						fillType = unit.fillType;
					end
					local info = self:getFillInfo(self:getFillTypeName(fillType), percentage, false, false, 1);
					if Inspector.display.variant == Inspector.display.VARIANT_FULL then
						info.draw = true;
					end
					table.insert(fill, info);
					draw = draw or info.draw;
				end
			end
		end
		if isSowingMachine then
			local spec = vehicle.spec_sowingMachine;
			local index = spec.fillUnitIndex;
			skipFillUnit[index] = true;
			local unit = vehicle:getFillUnitByIndex(index);
			local percentage = self:getFillUnitPercentage(unit);
			if percentage ~= nil then
				local fillType = FillType.SEEDS;
				if unit ~= nil and unit.fillType > 1 then
					fillType = unit.fillType;
				end
				local info = self:getFillInfo(self:getFillTypeName(fillType), percentage, false, false, 1, true);
				if Inspector.display.variant == Inspector.display.VARIANT_FULL then
					info.draw = true;
				end
				table.insert(fill, info);
				draw = draw or info.draw;
			end
		end
		if isSprayer then
			local spec = vehicle.spec_sprayer;
			skipFillUnit[spec.fillUnitIndex] = true;
			for _, sprayType in ipairs(spec.sprayTypes) do
				if sprayType ~= nil then
					skipFillUnit[sprayType.fillUnitIndex] = true;
				end
			end
			local index = vehicle:getSprayerFillUnitIndex();
			local unit = vehicle:getFillUnitByIndex(index);
			local percentage = self:getFillUnitPercentage(unit);
			if percentage ~= nil then
				local fillType = vehicle:getFillUnitLastValidFillType(index);
				if fillType == FillType.UNKNOWN then
					fillType = vehicle:getFillUnitFirstSupportedFillType(index)
				end
				local info = self:getFillInfo(self:getFillTypeName(fillType), percentage, false, false, 1, true);
				if Inspector.display.variant == Inspector.display.VARIANT_FULL then
					info.draw = true;
				end
				table.insert(fill, info);
				draw = draw or info.draw;
			end
		end
		for i,unit in ipairs(vehicle:getFillUnits()) do
			if (skipFillUnit[i] == nil) and (unit.fillType ~= 32 or not isMotorized) and unit.fillType ~= 33 and unit.fillType ~= 34 then
				local percentage = self:getFillUnitPercentage(unit);
				if percentage ~= nil then
					local warnOnHigh = nil;
					if isCombine then
						warnOnHigh = true;
					end
					local info = self:getFillInfo(self:getFillTypeName(unit.fillType), percentage, true, warnOnHigh, 1, false);
					if info.percentage > 0.5 then
						info.draw = true;
					elseif Inspector.display.variant == Inspector.display.VARIANT_FULL then
						info.draw = true;
					end
					table.insert(fill, info);
					draw = draw or info.draw;
				end
			end
		end
	end
	
	return draw, fill;
end

function Inspector:renderVehicleInfo(info)
	local first, shouldDraw = true, info.isFarmControlled or (info.owned and info.isControlled);
	for i,part in ipairs(info.parts) do
		if part.draw or first or shouldDraw then
			self.nLine = self.nLine + 1
			local x, y = Inspector.pos[1], Inspector.pos[2] - (self.nLine * Inspector.font.size * Inspector.font.line);

			local bold, warning, urgent = part.isSelfControlled, false, false;
			local line = part.name;
			if first and info.isAIBlocked then
				line = line .. " " .. g_i18n:getText("text_Inspector_AI_blocked");
				bold = true;
				if Inspector.AI.blocked.urgent then
					urgent = true;
				else
					warning = true;
				end
			end
			for j,fill in ipairs(part.fill) do
				if fill.draw or (fill.isMachineFill and shouldDraw) then
					bold = bold or fill.bold;
					warning = warning or fill.warning;
					urgent = urgent or fill.urgent;
					if fill.name == "Unknown" then
						line = line .. " " .. g_i18n:getText("text_Inspector_empty");
					else
						line = line .. " " .. fill.name .. "@" .. string.format("%.0f", fill.percentage) .. "%";
					end
				end
			end
			
			setTextBold(bold);
			
			if part.actor ~= nil then
				if part.isFarmControlled then
					setTextColor(unpack(Inspector.color.owned.normal));
				else
					setTextColor(unpack(Inspector.color.other.normal));
				end
				setTextAlignment(RenderText.ALIGN_RIGHT);
				renderText(x, y, Inspector.font.size, part.actor .. " ");
				setTextAlignment(RenderText.ALIGN_LEFT);
			end
			
			local colorPath;
			if part.owned then
				colorPath = Inspector.color.owned;
			else
				colorPath = Inspector.color.other;
			end
			if urgent then
				setTextColor(unpack(colorPath.urgent));
			elseif warning then
				setTextColor(unpack(colorPath.warning));
			else
				setTextColor(unpack(colorPath.normal));
			end

			if first then
				first = false;
			else
				line = "  " .. line;
			end
			
			renderText(x, y, Inspector.font.size, line);
		end
	end
end

function Inspector:onAIStart()
	self.Inspector_AIblocked = false;
end

function Inspector:onAIBlock()
	self.Inspector_AIblocked = true;
end

function Inspector:onAIContinue()
	self.Inspector_AIblocked = false;
end

if g_client ~= nil then
	-- client side mod only
	addModEventListener(Inspector);
	FSBaseMission.registerActionEvents = Utils.appendedFunction(FSBaseMission.registerActionEvents, Inspector.registerActionEvents);
	AIVehicle.registerEventListeners = Utils.appendedFunction(AIVehicle.registerEventListeners, Inspector.registerEventListeners);
end
